import bpy
from bpy.app import handlers

from .utils.draw_handles import DrawHandles
from .props.section_box import *

# from .update import get_update_popup
from .utils.properties import prop_filter
from .utils.modals import init_modals
from .utils.convert_old import convert_old
from .utils.addon import addon_enable, addon_disable
from . import addon_id


@handlers.persistent
def update_handler(dummy):
    # show update popup
    pass
    # if get_update_popup():
    #     getattr(bpy.ops, addon_id).update_check("INVOKE_DEFAULT", ask_check=True)


@handlers.persistent
def init(dummy):
    if not bpy.context.scene.get("sectionbox"):
        return

    # init draw handles
    if not bpy.app.driver_namespace.get("sectionbox.draw_handles"):
        bpy.app.driver_namespace["sectionbox.draw_handles"] = DrawHandles(bpy.context)

    # init modals
    init_modals(bpy.context)


@handlers.persistent
def convert(filepath):
    addon_enable("Section Box")

    context = bpy.context
    for scene in bpy.data.scenes:
        with context.temp_override(scene=scene):
            convert_old(context)

    addon_disable("Section Box")


@handlers.persistent
def store_props(scene):
    for sb in [i.object.sectionbox for i in scene.sectionbox.list]:
        for prop in filter(lambda a: prop_filter(sb, a), dir(sb.pre_settings)):
            try:
                setattr(sb.pre_settings, prop, getattr(sb, prop))
            except:
                continue


@handlers.persistent
def update_props(scene):
    for sb in [i.object.sectionbox for i in scene.sectionbox.list]:
        for prop in [
            p for p in filter(lambda a: prop_filter(sb, a), dir(sb.pre_settings))
        ]:
            value = getattr(sb, prop)
            pre_value = getattr(sb.pre_settings, prop)
            if value == pre_value:
                continue

            if hasattr(value, "__iter__"):
                if value[:] == pre_value[:]:
                    continue

            try:
                setattr(sb, prop, value)
            except:
                continue
            else:
                setattr(sb.pre_settings, prop, value)

        pre_plane = sb.active_plane
        for plane in sb.planes:
            sb.active_plane = plane.id
            for prop in [
                p
                for p in filter(
                    lambda a: prop_filter(plane, a), dir(plane.pre_settings)
                )
            ]:
                value = getattr(plane, prop)
                pre_value = getattr(plane.pre_settings, prop)
                if value == pre_value:
                    continue

                if hasattr(value, "__iter__"):
                    if value[:] == pre_value[:]:
                        continue

                try:
                    setattr(plane, prop, value)
                except:
                    continue
                else:
                    setattr(plane.pre_settings, prop, value)

        sb.active_plane = pre_plane
