import bpy
from bpy.types import Operator

from ..utils.sectionbox import get_sectionbox


class SECTIONBOX_OT_CrossSection(Operator):
    bl_idname = "sectionbox.cross_section"
    bl_label = "Cross Section"
    bl_description = "Create a cross section from this section plane"
    bl_options = {"UNDO", "INTERNAL"}

    action: bpy.props.StringProperty()
    enabled: bpy.props.BoolProperty()

    prev_active_camera = None
    lineset_settings = {}
    prev_render_settings = {}

    @classmethod
    def poll(cls, context):
        return True

    @classmethod
    def description(cls, context, properties):
        if properties.enabled:
            if properties.action == "create":
                return "Create a cross section from this section plane"
            elif properties.action == "remove":
                return "Remove the cross section of this section plane"
            elif properties.action == "update":
                return "Update the cross section of this section plane"
        else:
            return "Plane does not intersect with any objects"

        return cls.bl_description

    def execute(self, context):
        sb = get_sectionbox(context)
        if not sb:
            return {"CANCELLED"}

        plane = sb.planes[sb.active_plane]
        if self.action == "create":
            if not self.create(context, sb, plane):
                return {"CANCELLED"}
        elif self.action == "remove":
            plane.delete_cross_section(context)
            if sb.elevation_create_link:
                bpy.ops.sectionbox.elevation("INVOKE_DEFAULT", action="remove")
        elif self.action == "update":
            if not self.create(context, sb, plane):
                return {"CANCELLED"}

        action_txt = self.action + "d"
        if plane.elevation:
            text = f"Cross section and elevation {action_txt}!"
        else:
            text = f"Cross section {action_txt}!"
        self.report({"INFO"}, message=text)

        return {"FINISHED"}

    def create(self, context, sb, plane):
        prev_ele = bool(plane.elevation)
        plane.delete_cross_section(context)
        if not sb.cross_section(context, plane):
            self.report({"ERROR"}, "No cross section possible! Make sure that the plane is intersecting objects.")
            return None

        if prev_ele or sb.elevation_create_link:
            try:
                bpy.ops.preferences.addon_enable(module="render_freestyle_svg")
            except:
                self.report({"ERROR"}, "Freestyle SVG Exporter addon not found!")
                return None

            bpy.ops.sectionbox.elevation("INVOKE_DEFAULT", action="create")
            self.report({"INFO"}, "Cross section and elevation created!")

        else:
            self.report({"INFO"}, "Cross section created!")

        return True
