import bpy
from mathutils import Vector

from ..utils.sectionbox import get_sectionbox


class SECTIONBOX_OT_HoverModal(bpy.types.Operator):
    bl_idname = "sectionbox.hover_modal"
    bl_label = "Hover Modal"
    bl_options = {"INTERNAL"}

    prev_plane = None
    prev_edge_id = None

    @classmethod
    def poll(cls, context):
        return True

    def invoke(self, context, event):
        context.window_manager.modal_handler_add(self)
        return {"RUNNING_MODAL"}

    def modal(self, context, event):
        if not context.window_manager.sectionbox.running_modals:
            return {"CANCELLED"}

        if not context.area:
            for a in context.screen.areas:
                if a.type == "VIEW_3D":
                    for r in a.regions:
                        if r.type == "WINDOW":
                            with context.temp_override(area=a, region=r):
                                self.hover_check(context, event)

                            break

                    break
        else:
            self.hover_check(context, event)

        return {"PASS_THROUGH"}

    def get_mouse_cos(self, context, event):
        x = event.mouse_x - context.region.x
        y = event.mouse_y - context.region.y
        return Vector((x, y))

    def hover_check(self, context, event):
        sb = get_sectionbox(context, check_active=True)
        if not sb:
            return

        self.mouse_start = self.get_mouse_cos(context, event)
        if not context.window_manager.sectionbox.edge_select:
            self.plane = sb.hit_plane(context, self.mouse_start, select=False)
            if self.plane is not self.prev_plane:
                if self.plane is None:
                    context.window_manager.sectionbox.hovering_plane = -1
                else:
                    context.window_manager.sectionbox.hovering_plane = self.plane.id

                context.area.tag_redraw()
                self.prev_plane = self.plane
        else:
            plane = sb.planes[sb.active_plane]
            edge_id = plane.select_edge(context, self.mouse_start)
            if edge_id is not self.prev_edge_id:
                if edge_id is None:
                    context.window_manager.sectionbox.hovering_edge = -1
                else:
                    context.window_manager.sectionbox.hovering_edge = edge_id

                context.area.tag_redraw()
                self.prev_edge_id = edge_id
