import bpy

from ..utils.ui import draw_icon


class BLENDAI_OT_PopupBase(bpy.types.Operator):
    bl_idname = "blendai.popup_base"
    bl_label = ""
    bl_options = {"INTERNAL"}

    action: bpy.props.StringProperty(default="init", options={"HIDDEN", "SKIP_SAVE"})

    icon: str = None
    dialog = False
    persistent = False
    # if persistent
    _closed = False
    _popup_location = None

    @property
    def closed(self):
        return self.__class__._closed

    @closed.setter
    def closed(self, value):
        self.__class__._closed = value

    @property
    def popup_location(self):
        return self.__class__._popup_location

    @popup_location.setter
    def popup_location(self, value):
        self.__class__._popup_location = value

    def execute(self, context):
        return {"FINISHED"}

    def invoke(self, context, event):
        self.popup_location = event.mouse_x, event.mouse_y
        if event.type in {"ESC", "LEFT_ARROW"} and self.persistent:
            self.closed = True
            return {"CANCELLED"}

        if self.action == "init":
            wm = context.window_manager
            return (
                wm.invoke_props_dialog(self, width=300)
                if self.dialog
                else wm.invoke_popup(self)
            )

    def cancel(self, context):
        if not self.persistent:
            return

        if self.closed:
            return

        context.window.cursor_warp(*self.popup_location)
        getattr(bpy.ops.blendai, self.__class__.bl_idname.split(".")[-1])(
            "INVOKE_DEFAULT", **self.as_keywords()
        )

    def draw(self, context):
        if self.icon is None:
            return

        layout = self.layout
        draw_icon(layout, self.icon, spacing_top=1)
