import bpy
from bpy.types import Operator

from . import addon_id


class Support(Operator):
    bl_idname = "sectionbox.support"
    bl_label = ""
    bl_options = {"INTERNAL"}

    url: bpy.props.StringProperty(options={"HIDDEN"})
    type: bpy.props.EnumProperty(
        name="Type",
        items=[
            ("Question", "Ask a Question", "", "QUESTION", 0),
            ("Bug", "Report a Bug", "", "ERROR", 1),
            ("Request", "Request a feature", "", "OUTLINER_DATA_LIGHT", 2),
        ],
    )

    def draw(self, context):
        layout = self.layout
        row = layout.row()
        row.alignment = "CENTER"
        row.label(text="I want to")
        row.prop(self, "type", text="")

        layout.separator()
        row = layout.row()
        row.alignment = "CENTER"
        text = []
        if self.type == "Question":
            text.append("To best help you please include")
            text.append("the following information in your message:")

        elif self.type == "Bug":
            text.append("To fix it as fast as possible please include")
            text.append("the following information in your message:")

        elif self.type == "Request":
            text.append("To best implement it please include")
            text.append("the following information in your message:")

        row.label(text=text[0])
        for t in text[1:]:
            row = layout.row()
            row.alignment = "CENTER"
            row.scale_y = 0.3
            row.label(text=t)

        layout.separator()
        row = layout.row()
        row.alignment = "CENTER"
        box = row.box()
        if self.type == "Question":
            col = box.column()
            col.label(text="Name of the addon", icon="DOT")

        elif self.type == "Bug":
            col = box.column()
            col.label(text="Name of the addon", icon="DOT")
            col.scale_y = 0.8
            col.label(text="Addon version", icon="DOT")
            col.scale_y = 0.8
            col.label(text="Blender version", icon="DOT")
            col.scale_y = 0.8
            col.label(text="Operating system", icon="DOT")
            col.scale_y = 0.8
            col.label(text="Feature used", icon="DOT")
            col.scale_y = 0.8
            col.label(text="Error message if any", icon="DOT")
            col.scale_y = 0.8
            col.label(text="File if possible", icon="DOT")

        elif self.type == "Request":
            col = box.column()
            col.label(text="Name of the addon", icon="DOT")
            col.scale_y = 0.8
            col.label(text="How it should work", icon="DOT")
            col.scale_y = 0.8
            col.label(text="Example if possible", icon="DOT")

        layout.separator(factor=1.5)
        row = layout.row()
        row.alignment = "CENTER"
        row.label(text="Thank you!")
        row = layout.row()
        row.alignment = "CENTER"
        row.scale_y = 0.3
        row.label(text="I will get back to you as soon as possible.")
        layout.separator(factor=2)

    def invoke(self, context, event):
        return context.window_manager.invoke_props_dialog(self, width=300)

    def execute(self, context):
        import webbrowser

        webbrowser.open(self.url)
        return {"FINISHED"}
