import pkg_resources
import requests
import os
import sys
import subprocess

import bpy

from ..utils.resources import get_modules_path


def get_os():
    return bpy.app.build_platform.decode("utf-8")


def internet_connection():
    try:
        requests.get("https://www.rubenmesserschmidt.com/", timeout=5)
    except Exception:
        return False

    return True


def download_file(url, filepath):
    response = requests.get(url, stream=True)
    if response.status_code == 200:
        with open(filepath, "wb") as file:
            for chunk in response.iter_content(1024):
                file.write(chunk)


def is_module_installed(module):
    try:
        pkg_resources.get_distribution(module)
        return True
    except pkg_resources.DistributionNotFound:
        return False


def install_module(module: str):
    if is_module_installed(module):
        return True

    modules_dir = get_modules_path()
    cmd = [
        sys.executable,
        "-m",
        "pip",
        "install",
        module,
        "--target",
        os.fspath(modules_dir),
    ]
    try:
        subprocess.check_call(cmd)

    except subprocess.SubprocessError as e:
        print(f"Exception: {e}")
        return False

    return True


def get_missing_modules(modules):
    return [module for module in modules if not is_module_installed(module)]
